import ij.*;
import ij.process.*;
import ij.gui.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import ij.plugin.*;
import ij.plugin.frame.*;

/* step through a N dimentional dataset one dimension at a time.
 */
public class Frame_Slider implements PlugIn, DialogListener, ItemListener {

   TreeMap<String,TreeMap<Double,Double>> vars = null;
   TreeMap<String,Label> valLabels = new TreeMap<String,Label>();
   Label zLabel = null;
   ImagePlus imp = null;
   Panel panel = new Panel();

   public void run(String arg) {
      imp = IJ.getImage();

      GenericDialog gd = new NonBlockingGenericDialog("Frame_Slider");

      gd.addPanel(panel,GridBagConstraints.NORTH, new Insets(0, 0, 0, 0));
      panel.setLayout(new GridBagLayout());
      itemStateChanged(null);

      gd.addSlider("z",1,imp.getNSlices(),imp.getZ());
      gd.addToSameRow();
      gd.addMessage(""+imp.getZ());
      zLabel = (Label)gd.getMessage();
      zLabel.setFont(new Font("Monospaced",Font.BOLD,gd.getFont().getSize()));
      zLabel.setAlignment(Label.RIGHT);

      TreeMap<String,Double> cvv = F_Project.getVars(imp,imp.getT());
      if (cvv == null) {
         IJ.error(imp.getTitle()+"\n\nain't gonna happen...\nLook at the slice labels.");
         return;
         }
      vars = F_Project.getVars(imp);
      for(Map.Entry<String,TreeMap<Double,Double>> e : vars.entrySet()) {
         gd.addSlider(e.getKey(),0, e.getValue().size()-1,e.getValue().get(cvv.get(e.getKey())).intValue()-1);
         gd.addToSameRow();
         gd.addMessage(""+cvv.get(e.getKey()));
         Label l = (Label)gd.getMessage();
         l.setFont(new Font("Monospaced",Font.BOLD,gd.getFont().getSize()));
         l.setAlignment(Label.RIGHT);
         valLabels.put(e.getKey(),l);
         }
      
      gd.addDialogListener(this);
      (gd.getButtons()[0]).setEnabled(false);
      gd.showDialog();
      if (gd.wasCanceled())
         return;
      }

   Map<Choice,ImagePlus> ic = new HashMap<Choice,ImagePlus>();
   Map<Choice,Label>     il = new HashMap<Choice,Label>();
   public void itemStateChanged(ItemEvent e) {
//IJ.log("itemStateChanged "+e);
      Choice c = null;

      GridBagConstraints gbc = new GridBagConstraints();
      gbc.fill = GridBagConstraints.VERTICAL;
      gbc.gridx = 0;
      gbc.gridy = 0;
      gbc.weightx = 0.5;

      if (e == null) {
         c = impChoice(IJ.getImage());
         
         panel.add(c,gbc);
         ic.put(c,IJ.getImage());

         Label l = new Label("        ",Label.RIGHT);
         gbc.gridx = 1;
         panel.add(l,gbc);
         il.put(c,l);

         c = null;
         }
      else c = (Choice)e.getSource();
//IJ.log("Choice "+c+"  "+ic.get(c));

      if (c == null || ic.get(c) == null) {
         Choice cc = impChoice(null);
         gbc.gridx = 0;
         gbc.gridy = ic.size();
         panel.add(cc,gbc);
         ic.put(cc,null);

         Label l = new Label("        ",Label.RIGHT);
         gbc.gridx = 1;
         panel.add(l,gbc);
         il.put(cc,l);
         }

      //panel.revalidate();
      //panel.repaint();
      if (e != null) ((GenericDialog)panel.getParent()).pack();

      if (c == null) return;

      ic.put(c,(c.getSelectedIndex() > 0) ? WindowManager.getImage(imgIds[c.getSelectedIndex()-1]) : null);
      }

   int[] imgIds = WindowManager.getIDList();
   private Choice impChoice(ImagePlus imp) {
      Choice c = new Choice();
      c.add("<none>");
      c.select(0);
      for(int i=0; i<imgIds.length; i++) {
         ImagePlus tmpimp = WindowManager.getImage(imgIds[i]);
         c.add(tmpimp.getTitle());
         if (tmpimp == imp)
            c.select(i+1);
         }
      c.addItemListener(this);
      return c;
      }

   public boolean dialogItemChanged(GenericDialog gd, AWTEvent ev) {
      int z = (int)gd.getNextNumber();
      zLabel.setText(""+z);
      zLabel.setFont(new Font("Monospaced",Font.BOLD,gd.getFont().getSize()));

      TreeMap<String,Double> varval = new TreeMap<String,Double>();
      for(Map.Entry<String,TreeMap<Double,Double>> e : vars.entrySet()) {
         Double val = e.getValue().keySet().toArray(new Double[0])[(int)gd.getNextNumber()];
         varval.put(e.getKey(),val);
         valLabels.get(e.getKey()).setText(""+val);
         valLabels.get(e.getKey()).setFont(new Font("Monospaced",Font.BOLD,gd.getFont().getSize()));
         }

      for(ImagePlus imp : ic.values()) {
         if (imp == null) continue;
         imp.setZ(z);
         int f = F_Project.findFrame(imp,varval);
         if (f>0) imp.setT(f);
         imp.updateImage();
         }

      gd.pack();
      return true;
      }

}
